#ifndef _FAVIM_SOS_CONSTANTS_H_
#define _FAVIM_SOS_CONSTANTS_H_

#include <pid.h>

/**
 * @brief Minimum pid of core favim modules
 * 
 * Equal to APP_MOD_MIN_PID + 64 (192)
 */
#define FAVIM_CORE_MIN_PID (APP_MOD_MIN_PID + 64)
//! Maximum pid of core favim modules
#define FAVIM_CORE_MAX_PID (FAVIM_CORE_MIN_PID + 15)
/**
 * @brief Minimum pid of favim libraries
 * 
 * Equal to APP_MOD_MIN_PID (128)
 * We align them at 128 to make it easy to use masks
 */
#define FAVIM_LIB_MIN_PID APP_MOD_MIN_PID
/**
 * @brief Maximum pid of favim libraries
 * 
 * We limit the number of libraries to 64.
 */
#define FAVIM_LIB_MAX_PID (FAVIM_LIB_MIN_PID + 63)

//! PID's of core favim modules
enum 
{
	FAVIM_VIRUS_PID = FAVIM_CORE_MIN_PID + 0,
	FAVIM_STACK_PID = FAVIM_CORE_MIN_PID + 1,
	FAVIM_CAPSULE_PID = FAVIM_CORE_MIN_PID + 2,
	FAVIM_CAPSULE_MANAGER_PID = FAVIM_CORE_MIN_PID + 3,
	FAVIM_CONTEXT_PID = FAVIM_CORE_MIN_PID + 4,
	FAVIM_VM_PID = FAVIM_CORE_MIN_PID + 5,
	FAVIM_CONC_PID = FAVIM_CORE_MIN_PID + 6,
	FAVIM_LIB_PID = FAVIM_CORE_MIN_PID + 7,
	FAVIM_LIB_MANAGER_PID = FAVIM_CORE_MIN_PID + 8,
	FAVIM_SCHEDULER_PID = FAVIM_CORE_MIN_PID + 9,
	FAVIM_EVENT_MANAGER_PID = FAVIM_CORE_MIN_PID + 10,
};

//! PID's of the libs
enum
{
	FAVIM_LIB_BASICLIB_PID = FAVIM_LIB_MIN_PID + 0,
};

//! The start of the command ids in favim
#define FAVIM_CMD_START MOD_CMD_START
//! The start of the message ids in favim
#define FAVIM_MSG_START MOD_MSG_START

//! The command messages in favim
enum
{
	FAVIM_CMD_VM_ADD_CONTEXT = FAVIM_CMD_START + 0,
	FAVIM_CMD_VM_REMOVE_CONTEXT = FAVIM_CMD_START + 1,
	FAVIM_CMD_VM_EVENT_OCCURED = FAVIM_CMD_START + 2,
	FAVIM_CMD_VM_REBOOT = FAVIM_CMD_START + 3,
	FAVIM_CMD_LIB_EXECUTE = FAVIM_CMD_START + 4,
	FAVIM_CMD_LIB_MANAGER_ADD_LIBRARY = FAVIM_CMD_START + 5,
	FAVIM_CMD_LIB_MANAGER_REMOVE_LIBRARY = FAVIM_CMD_START + 6,
	FAVIM_CMD_VIRUS_VERSION_TIMER_TASK = FAVIM_CMD_START + 7,
	FAVIM_CMD_VIRUS_CAPSULE_TIMER_TASK = FAVIM_CMD_START + 8,
	FAVIM_CMD_VIRUS_CHECK_NEED_TASK = FAVIM_CMD_START + 9,
	FAVIM_CMD_SCHEDULER_HALT = FAVIM_CMD_START + 10,
	FAVIM_CMD_SCHEDULER_RESUME = FAVIM_CMD_START + 11,
	FAVIM_CMD_SCHEDULER_RUN_TASK = FAVIM_CMD_START + 12,
	FAVIM_CMD_CAPS_MANAGER_CAPS_ARRIVED = FAVIM_CMD_START + 13,
	FAVIM_CMD_VIRUS_CONTEXT_TIMER_TASK = FAVIM_CMD_START + 14,
};

//! The messages in favim
enum
{
	FAVIM_MSG_VIRUS_CAPSULECHUNKMSG = FAVIM_MSG_START + 0,
	FAVIM_MSG_VIRUS_CAPSULESTATUSMSG = FAVIM_MSG_START + 1,
	FAVIM_MSG_VIRUS_VERSIONMSG = FAVIM_MSG_START + 2,
	FAVIM_MSG_VIRUS_VERSIONREQUESTMSG = FAVIM_MSG_START + 3,
	FAVIM_MSG_VIRUS_CONTEXTMSG = FAVIM_MSG_START + 4,
};

//! The timers in favim
enum
{
//	SUBSCRIBE_TIMER = 0,
//	ERROR_TIMER = 1,
	FAVIM_VIRUS_VERSION_TIMER_ID,
	FAVIM_VIRUS_CAPSULE_TIMER_ID,
	FAVIM_EVENT_TIMER_ID,
	FAVIM_VIRUS_CONTEXT_TIMER_ID,
//	CLOCK_TIMER = 4,
};


#endif

