#ifndef _FAVIM_CONTEXT_H_
#define _FAVIM_CONTEXT_H_

//needed typedefs
typedef struct FavimContext FavimContext;
typedef enum FavimContextState FavimContextState;

#include "../favim.h"
#include "stack.h"
#include "../capsulestore/capsule.h"
#include "vm.h"
#include "../util/favim_queue.h"

#define FAVIM_LOCAL_COUNT 8

enum FavimContextState {
  FAVIM_CONTEXT_STATE_HALT,
  FAVIM_CONTEXT_STATE_WAITING,
  FAVIM_CONTEXT_STATE_READY,
  FAVIM_CONTEXT_STATE_RUN,
  FAVIM_CONTEXT_STATE_BLOCKED,
};


//! Structure representing a context in a FavimVM
struct FavimContext
{
	FavimCapsuleIndex pc;									//!< Current pc value
	FavimContextState state;								//!< Context state
	FavimContextId which;									//!< Context ID
	uint8_t heldSet[(FAVIM_GLOBAL_COUNT + 7) / 8];				//!< Held locks (bitmask)
	uint8_t releaseSet[(FAVIM_GLOBAL_COUNT + 7) / 8];	//!< Pending lock releases (bitmask)
	uint8_t acquireSet[(FAVIM_GLOBAL_COUNT + 7) / 8];	//!< Locks to acquire (bitmask)
	list_link_t link;										//!< Link entry for wait queues
	FavimQueue* queue;										//!< Current wait queue
	uint8_t num_executed;									//!< How long is the context executing
	FavimVM* vm;											//!< The VM to which this context belongs
	FavimVariable* locals[FAVIM_LOCAL_COUNT];									//!< The local variables of the context
	FavimCapsule* capsule;									//!< The capsule that is executing within this context
	FavimOperandStack* opstack;								//!< The operandstack of the context
} __attribute__((packed));


enum
{
	FAVIM_CONTEXT_NEW_FID,
	FAVIM_CONTEXT_RESET_FID,
};

typedef FavimContext* (*favim_context_new_proto)(func_cb_ptr, FavimVM*, FavimContextId);
typedef void (*favim_context_reset_proto)(func_cb_ptr, FavimContext*);

#endif
