/**
 * @file capsule.c
 * @brief The implementation of the capsule interface
 * @author Wouter Horr&eacute;
 */

#include <module.h>


#include <capsulestore/capsule.h>

// Forward declarations
static int8_t module(void *state, Message *e);
static FavimCapsule* new(func_cb_ptr);
static void finalize(func_cb_ptr, FavimCapsule*);

static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = FAVIM_CAPSULE_PID,
	.state_size     = sizeof(capsule_state_t),
	.platform_type  = HW_TYPE /* or PLATFORM_ANY */,
	.processor_type = MCU_TYPE,
	.code_id			= ehtons(FAVIM_CAPSULE_PID),
	.num_timers     = 0,
	.num_sub_func   = 0,
	.num_prov_func  = 2,
	.module_handler = module,
	.funct = {
		{new, "avv0", FAVIM_CAPSULE_PID, FAVIM_CAPSULE_NEW_FID},
		{finalize, "vav1", FAVIM_CAPSULE_PID, FAVIM_CAPSULE_FINALIZE_FID},
	},
};


//! The main message handling function
static int8_t module(void *state, Message *msg)
{
    capsule_state_t *s = (capsule_state_t *) state;

    switch (msg->type){

        case MSG_INIT:
		{
			s->pid = msg->did;
			s->default_capsule[0] = 0x01; // require basiclib
			s->default_capsule[1] = 0x00; // halt
			return SOS_OK;
		}


        case MSG_FINAL:
        {
			return SOS_OK;
        }


        default:
			return -EINVAL;
	}

    return SOS_OK;
}

//! Create a new capsule
static FavimCapsule* new(func_cb_ptr proto)
{
	capsule_state_t* s = (capsule_state_t*) ker_get_module_state(FAVIM_CAPSULE_PID);
	
	DEBUG("New Capsule\n");
	int i;
	FavimCapsule* new = (FavimCapsule *) my_malloc(sizeof(FavimCapsule),FAVIM_CAPSULE_PID);
	if(new == NULL){
		//TODO error handling
		panic();
		return NULL;
	}
	new->version = 0;
	for(i=0; i < ((FAVIM_GLOBAL_COUNT + 7) / 8) ; ++i){
		new->usedVars[i] = 0;
	}
	new->dataSize=2;
//	new->data[0] = 0x01; //require basic lib
//	new->data[1] = 0x00; //halt
	new->data = s->default_capsule;
	return new;
}

//! Finalize a capsule
static void finalize(func_cb_ptr proto, FavimCapsule* capsule)
{
	capsule_state_t* s = (capsule_state_t*) ker_get_module_state(FAVIM_CAPSULE_PID);

	if(capsule->data != s->default_capsule){
		ker_free(capsule->data);
	}
	ker_free(capsule);
}


#ifndef _MODULE_
mod_header_ptr capsule_get_header()
{
    return sos_get_header_address(mod_header);
}
#endif

