/**
 * @file capsule_manager.h
 * @brief The interface of the FAViM Capsule Manager
 * @author Wouter Horr&eacute;
 */

#ifndef _FAVIM_CAPSULE_MANAGER_H_
#define _FAVIM_CAPSULE_MANAGER_H_

typedef struct FavimCapsuleMsg FavimCapsuleMsg;
typedef struct FavimCapsuleIdVersionPair FavimCapsuleIdVersionPair;

#include "../favim.h"
#include "capsule.h"
#include "../vmstore/vm_manager.h"

//! A Capsule ID
typedef uint8_t FavimCapsuleId;

struct FavimCapsuleMsg
{
	FavimCapsuleId id;
	FavimCapsule capsule;
} __attribute__((packed)) ;

struct FavimCapsuleIdVersionPair
{
	FavimCapsuleId id;
	FavimCapsuleVersion version;
} __attribute__((packed)) ;

//! The function id of the exported functions
enum
{
	FAVIM_CAPSULE_MANAGER_ADD_CAPSULE_FID,
	FAVIM_CAPSULE_MANAGER_REMOVE_CAPSULE_FID,
	FAVIM_CAPSULE_MANAGER_GET_CAPSULE_FID,
	FAVIM_CAPSULE_MANAGER_GET_NB_CAPSULES_FID,
	FAVIM_CAPSULE_MANAGER_GET_ALL_VERSIONS_FID,
};

// Typedefs needed to call the exported functions
typedef int8_t (*favim_capsule_manager_add_capsule_proto)(func_cb_ptr, FavimVMId, FavimContextId, FavimCapsule*);
typedef int8_t (*favim_capsule_manager_remove_capsule_proto)(func_cb_ptr, FavimVMId, FavimContextId);
typedef FavimCapsule * (*favim_capsule_manager_get_capsule_proto)(func_cb_ptr, FavimCapsuleId);
typedef uint8_t (*favim_capsule_manager_get_nb_capsules_proto)(func_cb_ptr);
typedef FavimCapsuleIdVersionPair* (*favim_capsule_manager_get_all_versions_proto)(func_cb_ptr);

typedef struct HashEntry HashEntry;

struct HashEntry {
	FavimCapsuleId id;
	FavimCapsule* capsule;
	HashEntry* next;
};

//! The state of the capsule manager
typedef struct {
	func_cb_ptr vm_halt;
	func_cb_ptr analyse_capsule;
	sos_pid_t pid;
	uint8_t nb_capsules;
	HashEntry * capsules[8];
} capsule_manager_state_t;



#endif
