#ifndef _FAVIM_EVENT_MANAGER_H_
#define _FAVIM_EVENT_MANAGER_H_

#include "../favim.h"
#include "../vmstore/vm_manager.h"
#include <sos_timer.h>

typedef uint8_t FavimEventId;
typedef uint8_t FavimTimerInterval;
typedef enum FavimEventType FavimEventType;
typedef enum FavimTimerType FavimTimerType;
typedef struct FavimEventTimer FavimEventTimer;

//! The types of events
enum FavimEventType
{
	FAVIM_EVENT_SINGLE_VM,
	FAVIM_EVENT_GLOBAL,
};

//! The types of timers
enum FavimTimerType
{
	FAVIM_TIMER_REPEAT = TIMER_REPEAT,
	FAVIM_TIMER_ONE_SHOT = TIMER_ONE_SHOT,
};

//! The function ids of the exported functions
enum
{
	FAVIM_EVENT_MANAGER_START_TIMER_FID,
	FAVIM_EVENT_MANAGER_STOP_TIMER_FID,
	FAVIM_EVENT_MANAGER_INIT_EVENT_FID,
	FAVIM_EVENT_MANAGER_TRIGGER_EVENT_FID,
	FAVIM_EVENT_MANAGER_SIGNAL_HALT_FID,
};	

//! Structure representing a favim timer
struct FavimEventTimer {
	FavimEventType type;
	FavimEventId id;
	FavimValueVariable interval;
	FavimValueVariable ticks;
	FavimEventTimer* next;
}__attribute__((packed));

typedef int8_t (*favim_event_manager_start_timer_proto)(func_cb_ptr,FavimEventType,FavimVMId,FavimEventId,FavimTimerType,FavimTimerInterval);
typedef int8_t (*favim_event_manager_stop_timer_proto)(func_cb_ptr,FavimEventType,FavimVMId,FavimEventId);
//typedef int8_t (*favim_event_manager_init_event_proto)(func_cb_ptr,FavimEventId);
typedef int8_t (*favim_event_manager_trigger_event_proto)(func_cb_ptr,FavimEventType,FavimVMId,FavimEventId);
typedef void (*favim_event_manager_signal_halt_proto)(func_cb_ptr,FavimVMId);


//! The state of the FAViM event manager
typedef struct {
	sos_pid_t pid;
	FavimEventTimer* timerlist;
//	uint8_t timercount;
} event_man_state_t;


#endif
