#ifndef _FAVIM_SOS_CONSTANTS_H_
#define _FAVIM_SOS_CONSTANTS_H_

#include <pid.h>

/**
 * @brief Minimum pid of core favim modules
 * 
 * Equal to APP_MOD_MIN_PID + 64 (192)
 */
#define FAVIM_CORE_MIN_PID (APP_MOD_MIN_PID + 64)
//! Maximum pid of core favim modules
#define FAVIM_CORE_MAX_PID (FAVIM_CORE_MIN_PID + 15)
/**
 * @brief Minimum pid of favim libraries
 * 
 * Equal to APP_MOD_MIN_PID (128)
 * We align them at 128 to make it easy to use masks
 */
#define FAVIM_LIB_MIN_PID APP_MOD_MIN_PID
/**
 * @brief Maximum pid of favim libraries
 * 
 * We limit the number of libraries to 64.
 */
#define FAVIM_LIB_MAX_PID (FAVIM_LIB_MIN_PID + 63)

//! PID's of core favim modules
enum 
{
	FAVIM_VIRUS_PID = FAVIM_CORE_MIN_PID,	//192
	FAVIM_STACK_PID,							//193
	FAVIM_CAPSULE_PID,						//194
	FAVIM_CAPSULE_MANAGER_PID,				//195
	FAVIM_CONTEXT_PID,						//196
	FAVIM_VM_PID,							//197
	FAVIM_CONC_PID,							//198
	FAVIM_LIB_PID,							//199
	FAVIM_LIB_MANAGER_PID,					//200
	FAVIM_SCHEDULER_PID,						//201
	FAVIM_EVENT_MANAGER_PID,					//202
	FAVIM_VM_MANAGER_PID,					//203
};

//! PID's of the libs
enum
{
	FAVIM_LIB_BASICLIB_PID = FAVIM_LIB_MIN_PID, //128
	FAVIM_LIB_TESTLIB_PID,						//129
};

//! The start of the command ids in favim
#define FAVIM_CMD_START MOD_CMD_START
//! The start of the message ids in favim
#define FAVIM_MSG_START MOD_MSG_START

//! The command messages in favim
enum
{
//	FAVIM_CMD_VM_ADD_CONTEXT = FAVIM_CMD_START,
//	FAVIM_CMD_VM_REMOVE_CONTEXT,
	FAVIM_CMD_VM_INSTALL_DESCRIPTION = FAVIM_CMD_START,
	FAVIM_CMD_VM_EVENT_OCCURED,
	FAVIM_CMD_VM_REBOOT,
	FAVIM_CMD_LIB_EXECUTE,
	FAVIM_CMD_LIB_MANAGER_ADD_LIBRARY,
	FAVIM_CMD_LIB_MANAGER_REMOVE_LIBRARY,
	FAVIM_CMD_VIRUS_CAPSULE_VERSION_TIMER_TASK,
	FAVIM_CMD_VIRUS_CAPSULE_TIMER_TASK,
	FAVIM_CMD_VIRUS_CAPSULE_CHECK_NEED_TASK,
	FAVIM_CMD_VIRUS_VM_VERSION_TIMER_TASK,
	FAVIM_CMD_VIRUS_VM_TIMER_TASK,
	FAVIM_CMD_SCHEDULER_HALT,
	FAVIM_CMD_SCHEDULER_RESUME,
	FAVIM_CMD_SCHEDULER_RUN_TASK,
	FAVIM_CMD_CAPS_MANAGER_CAPS_ARRIVED,
//	FAVIM_CMD_VM_MAN_ADD_VM,
//	FAVIM_CMD_VM_MAN_REMOVE_VM,
	FAVIM_CMD_VM_MAN_VMMSG_ARRIVED,
	FAVIM_CMD_VM_MAN_TRIGGER_GLOBAL,
};

//! The messages in favim
enum
{
	FAVIM_MSG_VIRUS_CAPSULECHUNKMSG = FAVIM_MSG_START,
	FAVIM_MSG_VIRUS_CAPSULESTATUSMSG,
	FAVIM_MSG_VIRUS_CAPSULE_VERSIONMSG,
//	FAVIM_MSG_VIRUS_CAPSULE_VERSIONREQUESTMSG,
	FAVIM_MSG_VIRUS_VMMSG,
	FAVIM_MSG_VIRUS_VM_VERSIONMSG,
};

//! The timers in favim
enum
{
//	SUBSCRIBE_TIMER = 0,
//	ERROR_TIMER = 1,
	FAVIM_VIRUS_CAPSULE_VERSION_TIMER_ID,
	FAVIM_VIRUS_CAPSULE_TIMER_ID,
	FAVIM_VIRUS_VM_VERSION_TIMER_ID,
	FAVIM_VIRUS_VM_TIMER_ID,
	FAVIM_EVENT_TIMER_ID,
//	CLOCK_TIMER = 4,
};


#endif

