#ifndef _FAVIM_CONTEXT_H_
#define _FAVIM_CONTEXT_H_

//needed typedefs
typedef struct FavimContext FavimContext;
typedef enum FavimContextState FavimContextState;

#include "../favim.h"
#include "stack.h"
#include "../capsulestore/capsule.h"
#include "vm.h"
#include "vm_manager.h"
#include "../util/favim_queue.h"

#define FAVIM_LOCAL_COUNT 2

enum FavimContextState {
  FAVIM_CONTEXT_STATE_HALT,
  FAVIM_CONTEXT_STATE_WAITING,
  FAVIM_CONTEXT_STATE_READY,
  FAVIM_CONTEXT_STATE_RUN,
  FAVIM_CONTEXT_STATE_BLOCKED,
};


//! Structure representing a context in a FavimVM
struct FavimContext
{
	FavimVMId vm;											//!< The VM to which this context belongs
	FavimContextId which;									//!< Context ID
	FavimCapsuleIndex pc;									//!< Current pc value
	FavimContextState state;									//!< Context state
	uint8_t num_executed;									//!< How long is the context executing
	FavimVariable* locals[FAVIM_LOCAL_COUNT];				//!< The local variables of the context
	FavimCapsule* capsule;									//!< The capsule that is executing within this context
	FavimOperandStack* opstack;								//!< The operandstack of the context
	uint8_t heldSet[(FAVIM_GLOBAL_COUNT + 7) / 8];			//!< Held locks (bitmask)
	uint8_t releaseSet[(FAVIM_GLOBAL_COUNT + 7) / 8];		//!< Pending lock releases (bitmask)
	uint8_t acquireSet[(FAVIM_GLOBAL_COUNT + 7) / 8];		//!< Locks to acquire (bitmask)
	list_link_t link;										//!< Link entry for wait queues
	FavimQueue* queue;										//!< Current wait queue
} __attribute__((packed));


enum
{
	FAVIM_CONTEXT_NEW_FID,
	FAVIM_CONTEXT_FINALIZE_FID,
	FAVIM_CONTEXT_RESET_FID,
};

typedef FavimContext* (*favim_context_new_proto)(func_cb_ptr, FavimVMId, FavimContextId);
typedef void (*favim_context_finalize_proto)(func_cb_ptr, FavimContext*);
typedef void (*favim_context_reset_proto)(func_cb_ptr, FavimContext*);

//! The state of the context module
typedef struct {
	func_cb_ptr stack_new;
	func_cb_ptr stack_finalize;
	func_cb_ptr stack_reset;
	func_cb_ptr capsule_new;
	func_cb_ptr capsule_finalize;
	func_cb_ptr add_capsule;
	func_cb_ptr remove_capsule;
	sos_pid_t pid;
} context_state_t;

#endif
