#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <netdb.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/socket.h>

int sossrv_connect(char* server_addr, char* server_port);
void printHelp(char* binary);


typedef struct {
  unsigned char  did;                          //!< module destination id
  unsigned char  sid;                          //!< module source id
  unsigned short daddr;                        //!< node destination address
  unsigned short saddr;                        //!< node source address
  unsigned char  type;                         //!< module specific message type
  unsigned char  len;                          //!< payload length 
  unsigned char  data[128];                    //!< actual payload
} __attribute__ ((packed)) SOS_Message_t;

typedef struct {
  long int version;
  unsigned char capsule_number;
  unsigned char capsuleLength;
  unsigned char piece_number;
  unsigned char payload[16];
} __attribute__ ((packed)) Favim_Capsule_t;

typedef struct {
	unsigned char id;
	unsigned char lib_mapping[8];
	long int contexts;
} __attribute__((packed)) FavimVMDescription;

typedef struct {
	FavimVMDescription desc;
	long int version;
	unsigned char present;
} __attribute__((packed)) Favim_VMMsg_t;

#define FAVIM_CONTEXT_BITS 5
#define FAVIM_CAPSULE_HEADER_SIZE 7
#define SOS_MSG_HEADER_SIZE 8
#define FAVIM_VM_INVALID 0xff;

unsigned char *data_ptr;
int fd = 0;
unsigned char data[200];
unsigned int i, inc;

main( argc, argv )
int argc;
char **argv;
{
	if(argc < 2)
	{
		printHelp(argv[0]);
		return 0;
	}
	if(strcmp(argv[1],"--capsule")==0){
		//Send capsule
		if (argc < 6)
		{
			printf("Usage Error!\n");
			printHelp(argv[0]);
			return 1;
		}
		
		SOS_Message_t *sos_pkt = (SOS_Message_t *)data;
		Favim_Capsule_t *favim_capsule = (Favim_Capsule_t *)sos_pkt->data;
		
		data_ptr = favim_capsule->payload;
		FILE *in;
		char version_filename[50];
		int read, temp;
		unsigned char cap_nr;
		
		if (strcmp(argv[4], "reboot") == 0)
		{
			cap_nr = (char)0;
		}
		else if (strcmp(argv[4], "timer0") == 0)
		{
			cap_nr = (char)1;
		}
		else
		{
			cap_nr = (char)atoi(argv[4]);
		}
		
		cap_nr = (char) (cap_nr + (atoi(argv[3]) << FAVIM_CONTEXT_BITS));
		
		printf("Capsule id: %d\n", cap_nr);
		
		
		fd = sossrv_connect("127.0.0.1", argv[5]);
		
		in = fopen(argv[2], "r");
		if(in == NULL){
			printf("Could not open input file\n");
			return 1;
		}
		
		read = fread(data_ptr, 1, 100, in);
		if(read > 16){
			//aiai
			// TODO change this!
			printf("For now, only capsules <= 16 bytes are supported\n");
			return 1;
		}
		sos_pkt->did = 192;
		sos_pkt->sid = 192;
		sos_pkt->daddr = 0xFFFF;
		sos_pkt->saddr = 0xFFFE;
		sos_pkt->type = 32;
		sos_pkt->len = FAVIM_CAPSULE_HEADER_SIZE + 16;

		sprintf(version_filename, "capsule%d_version.txt", (int)cap_nr);
		favim_capsule->version = get_version(version_filename);
		favim_capsule->capsule_number = cap_nr;
		favim_capsule->piece_number = 0;
		favim_capsule->capsuleLength = read;

		for(temp = 0; temp < FAVIM_CAPSULE_HEADER_SIZE+read; temp++) 
			printf("%x ", data[SOS_MSG_HEADER_SIZE+temp]);
		printf("\n");
	
		write(fd, data, SOS_MSG_HEADER_SIZE + FAVIM_CAPSULE_HEADER_SIZE + 16);
		close(fd);
				
	} else if(strcmp(argv[1],"--vm")==0){
		//Send vm
		if(argc < 4){
			printf("Usage Error!\n");
			printHelp(argv[0]);
			return 1;
		}
		int read;
		FILE *in;
		SOS_Message_t *sos_pkt = (SOS_Message_t *)data;
		Favim_VMMsg_t *favim_vmmsg = (Favim_VMMsg_t *) sos_pkt->data;
		char version_filename[50];

		sos_pkt->did = 192;
		sos_pkt->sid = 192;
		sos_pkt->daddr = 0xFFFF;
		sos_pkt->saddr = 0xFFFE;
		sos_pkt->type = 35;
		sos_pkt->len = sizeof(Favim_VMMsg_t);
		
		favim_vmmsg->desc.id = atoi(argv[3]);
		
		in = fopen(argv[2], "r");
		if(in == NULL){
			printf("Could not open input file\n");
			return 1;
		}
		read = fread(favim_vmmsg->desc.lib_mapping, 1, 12, in);		
		if(read != 12){
			printf("Input file invalid\n");
			return 1;
		}
		
		
		sprintf(version_filename, "vm%d_version.txt", (int)favim_vmmsg->desc.id);
		favim_vmmsg->version = get_version(version_filename);
		favim_vmmsg->present = 1;		
		fd = sossrv_connect("127.0.0.1", "7915");
		write(fd, data, SOS_MSG_HEADER_SIZE + sizeof(Favim_VMMsg_t));
		close(fd);
	} else if (strcmp(argv[1],"--vmdel")==0){
		//delete a vm
		if(argc < 3){
			printf("Usage Error!\n");
			printHelp(argv[0]);
			return 1;
		}
		SOS_Message_t *sos_pkt = (SOS_Message_t *)data;
		Favim_VMMsg_t *favim_vmmsg = (Favim_VMMsg_t *) sos_pkt->data;
		char version_filename[50];

		sos_pkt->did = 192;
		sos_pkt->sid = 192;
		sos_pkt->daddr = 0xFFFF;
		sos_pkt->saddr = 0xFFFE;
		sos_pkt->type = 35;
		sos_pkt->len = sizeof(Favim_VMMsg_t);
		
		favim_vmmsg->desc.id = atoi(argv[2]);
		favim_vmmsg->present = 0;

		sprintf(version_filename, "vm%d_version.txt", (int)favim_vmmsg->desc.id);
		favim_vmmsg->version = get_version(version_filename);
		
		fd = sossrv_connect("127.0.0.1", "7915");
		write(fd, data, SOS_MSG_HEADER_SIZE + sizeof(Favim_VMMsg_t));
		close(fd);
	} else {
		printf("Usage Error!\n");
		printHelp(argv[0]);
		return 1;
	}
}

void printHelp(char* binary)
{
	printf("Usage:\n");
	printf("Inject a capsule:\n");
	printf("%s --capsule <filename> <vm> <context> <port>\n", binary);
	printf("\t<filename> = input file containing compiled bytecode\n");
	printf("\t<vm> = id of the vm\n");
	printf("\t<context> = reboot | timer0 | (int)contextId\n");
	printf("\t<port> = portnumber to connect to sosserver\n\n");
	printf("Add or change a VM:\n");
	printf("%s --vm <filename> <vm>\n", binary);
	printf("\t<filename> = input file containing compiled description of VM\n");
	printf("\t<vm> = id of the vm\n\n");
	printf("Delete a VM:\n");
	printf("%s --vmdel <vm>\n", binary);
	printf("\t<vm> = id of the vm\n\n");
}

int sossrv_connect(char* server_addr, char* server_port)
{
	int clientsockfd;
	struct sockaddr_in server_address;

	// Create the socket
	if ((clientsockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		printf("Error: cannot open socket");
		exit(1);
	}

	// Setup the internet address
	server_address.sin_family = AF_INET;
	server_address.sin_port = htons(atoi(server_port));
	inet_aton(server_addr, &(server_address.sin_addr));
	memset(&(server_address.sin_zero), '\0', 8);
  
	printf("Connecting to server at %s\n", inet_ntoa(server_address.sin_addr));
	printf("Connecting to port at %d\n", ntohs(server_address.sin_port));
  
	// Connect
	if (connect(clientsockfd, (struct sockaddr *)&server_address, sizeof(struct sockaddr)) == -1) 
	{
		printf("Error: cannot connect to socket");
    	exit(1);
	}
	printf("Established TCP connection ..\n");
	
	return clientsockfd;
}

int get_version(char * version_filename){
		FILE *version_file;
		int version;

		version_file = fopen(version_filename, "r");
		if(version_file == NULL)
		{
			printf("New version file created\n");
			version = 0;
		}
		else
		{
			if( fscanf(version_file, "%ld", &version) > 0 )
			{
				printf("version = %ld\n", version);
			}
			else
			{
				printf("Nothing read from version file\n");
				version = 0;
			}
			fclose(version_file);
		}
		
		version_file = fopen(version_filename, "w");
		
		printf("writing %ld to file\n", version+1);
		fprintf(version_file, "%ld", ++version);
		
		fclose(version_file);
		return version;
}
