%{
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>

int state;
int locallib;

enum {
	FOUND_NOTHING,
	FOUND_LIB,
	FOUND_LOCAL_LIB,
	FOUND_CONTEXT,
};

#define MAX_LOCAL_LIB 7
#define MAX_CONTEXT 32
#define MAX_GLOBAL_LIB 63

typedef struct {
	unsigned char lib_mapping[8];
	long int contexts;
} __attribute__((packed)) FavimVMDescriptionPart;


FavimVMDescriptionPart desc;

%}

%%
LIB { state = FOUND_LIB; }
CONTEXT { state = FOUND_CONTEXT; }
[\-]*[0-9]+ {
	if(state == FOUND_LIB) {
		locallib = atoi(yytext);
		if(locallib <= MAX_LOCAL_LIB){
			state = FOUND_LOCAL_LIB;
		} else {
			printf("LIB locallibid globallibid: locallibid must be <= %i\n", MAX_LOCAL_LIB);
			state = FOUND_NOTHING;
		}
	} else if (state == FOUND_LOCAL_LIB) {
		int globallib = atoi(yytext);
		if(globallib <= MAX_GLOBAL_LIB){
			desc.lib_mapping[locallib] = globallib;
		} else {
			printf("LIB locallibid globallibid: globallidid must be <= %i\n", MAX_GLOBAL_LIB);
		}
		state = FOUND_NOTHING;
	} else if(state == FOUND_CONTEXT) {
		int con = atoi(yytext);
		if(con > MAX_CONTEXT) {
			printf("CONTEXT id: id must be <= %i\n", MAX_CONTEXT);
			state = FOUND_NOTHING;
		} else {
			desc.contexts |= (0x1 << con);
		}
	}
}

[ \t\n]
%%

main( argc, argv )
int argc;
char **argv;
{
	int i;
	FILE *out;
	
	for(i=0; i <= MAX_LOCAL_LIB; ++i){
		desc.lib_mapping[i] = 0xff;
	}
	desc.contexts = 0;
	
	if(argc < 3){
		printf("Usage Error!\n");
		printf("%s <inputfile> <outputfile>\n", argv[0]);
		return 1;
	}
	
	yyin = fopen( argv[1], "r" );
	if(yyin == NULL)
	{
		printf("Data file could not be opened\n");
		return 1;
	}
	
	out = fopen(argv[2], "w");

	yylex();

	fwrite(&desc, 1, 12, out);
	fclose(out);
}
	