/**
 * @file concurrency.c
 * @brief The concurrency manager of FAViM
 * @author Wouter Horr&eacute;
 */
 
#include <module.h>
#include <vmstore/concurrency.h>
#include <engine/scheduler.h>

//! The state of the concurrency manager
typedef struct {
	func_cb_ptr scheduler_submit;
	sos_pid_t pid;
} concurrency_state_t;

// Forward declarations
static int8_t module(void *state, Message *e);
static int8_t submit(func_cb_ptr, FavimContext*);

//! The module header of the concurrency manager
static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = FAVIM_CONC_PID,
	.state_size     = sizeof(concurrency_state_t),
    .num_timers     = 0,
	.num_sub_func   = 1,
	.num_prov_func  = 1,
	.module_handler = module,
	.funct = {
		//TODO error handler
		{NULL, "cav1", FAVIM_SCHEDULER_PID, FAVIM_SCHEDULER_SUBMIT_FID},
		{submit, "cav1", FAVIM_CONC_PID, FAVIM_CONC_SUBMIT_FID},
	},
};

//! The main message handling function
static int8_t module(void *state, Message *msg)
{
    concurrency_state_t *s = (concurrency_state_t *) state;

    switch (msg->type){

        case MSG_INIT:
		{
			s->pid = msg->did;
			return SOS_OK;
		}

        case MSG_FINAL:
        {
			return SOS_OK;
        }

        default:
			return -EINVAL;
	}

    return SOS_OK;
}

static int8_t submit(func_cb_ptr proto, FavimContext* context)
{
	concurrency_state_t * s = (concurrency_state_t*) ker_get_module_state(FAVIM_CONC_PID);
	
	// just passing to the scheduler for now
	SOS_CALL(s->scheduler_submit, favim_scheduler_submit_proto, context);
}

#ifndef _MODULE_
mod_header_ptr concurrency_get_header()
{
    return sos_get_header_address(mod_header);
}
#endif

