/**
 * @file context.c
 * @brief A execution context in a FAViM virtual machine
 * @author Wouter Horr&eacute;
 */
 
#include <module.h>
#include <vmstore/context.h>

//! The state of the context module
typedef struct {
	sos_pid_t pid;
} context_state_t;

//TODO some kind of init

// Forward declarations
static int8_t module(void *state, Message *e);
static void set_pc(func_cb_ptr, FavimContext*, FavimCapsuleIndex);
static void set_local(func_cb_ptr, FavimContext*, FavimVariableId, FavimVariable*);
static void set_capsule(func_cb_ptr, FavimContext*, FavimCapsule*);
//op niveau individuele lock?
static void set_use_set(func_cb_ptr, FavimContext*, uint8_t*);
static void set_acquire_set(func_cb_ptr, FavimContext*, uint8_t*);
static void set_release_set(func_cb_ptr, FavimContext*, uint8_t*);
//
static void reset(func_cb_ptr, FavimContext*);

//! The module header of the context module
static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = FAVIM_CONTEXT_PID,
	.state_size     = sizeof(context_state_t),
    .num_timers     = 0,
	.num_sub_func   = 0,
	.num_prov_func  = 7,
	.module_handler = module,
	.funct = {
		{set_pc, "vay2", FAVIM_CONTEXT_PID, FAVIM_CONTEXT_SET_PC_FID},
		{set_local, "vay3", FAVIM_CONTEXT_PID, FAVIM_CONTEXT_SET_LOCAL_FID},
		{set_capsule, "vaa2", FAVIM_CONTEXT_PID, FAVIM_CONTEXT_SET_CAPSULE_FID},
		{set_use_set, "vaE2", FAVIM_CONTEXT_PID, FAVIM_CONTEXT_SET_USE_SET_FID},
		{set_acquire_set, "vaE2", FAVIM_CONTEXT_PID, FAVIM_CONTEXT_SET_ACQUIRE_SET_FID},
		{set_release_set, "vaE2", FAVIM_CONTEXT_PID, FAVIM_CONTEXT_SET_RELEASE_SET_FID},
		{reset, "vav1", FAVIM_CONTEXT_PID, FAVIM_CONTEXT_RESET_FID},
	},
};

//! The main message handling function
static int8_t module(void *state, Message *msg)
{
    context_state_t *s = (context_state_t *) state;

    switch (msg->type){

        case MSG_INIT:
		{
			s->pid = msg->did;
			return SOS_OK;
		}


        case MSG_FINAL:
        {
			return SOS_OK;
        }

        default:
			return -EINVAL;
	}

    return SOS_OK;
}

//! Set the program counter
static void set_pc(func_cb_ptr proto, FavimContext* context, FavimCapsuleIndex pc)
{
	//TODO implementation
}

//! Set a variable local to the context
static void set_local(func_cb_ptr proto, FavimContext* context, FavimVariableId id, FavimVariable* value)
{
	//TODO implementation
}

//! Set the capsule executing in this context
static void set_capsule(func_cb_ptr proto, FavimContext* context, FavimCapsule* capsule)
{
	//TODO implementation
}

//op niveau individuele lock?
//! Set the set of used locks
static void set_use_set(func_cb_ptr proto, FavimContext* context, uint8_t* set)
{
	//TODO implementation
}

//! Set the set of locks to acquire
static void set_acquire_set(func_cb_ptr proto, FavimContext* context, uint8_t* set)
{
	//TODO implementation
}

//! Set the set of locks to release
static void set_release_set(func_cb_ptr proto, FavimContext* context, uint8_t* set)
{
	//TODO implementation
}

//! Reset the given context
static void reset(func_cb_ptr proto, FavimContext* context)
{
	//TODO implementation
}

#ifndef _MODULE_
mod_header_ptr context_get_header()
{
    return sos_get_header_address(mod_header);
}
#endif

