/**
 * @file stack.c
 * @brief The implementation of the FAViM stack interface
 * @author Wouter Horr&eacute;
 */
#include <module.h>

#include <vmstore/stack.h>

/// The state of the stack module
typedef struct {
	sos_pid_t pid;
} stack_state_t;

//forward declarations
static int8_t module(void *state, Message *e);
static int8_t stack_push(func_cb_ptr proto, FavimOperandStack* stack, FavimVariable var);
static FavimVariable* stack_pop(func_cb_ptr proto, FavimOperandStack* stack);
static int8_t stack_reset(func_cb_ptr proto, FavimOperandStack* stack);

static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = FAVIM_STACK_PID,
	.state_size     = sizeof(stack_state_t),
	.num_timers     = 0,
	.num_sub_func   = 0,
	.num_prov_func  = 3,
	.module_handler = module,
	.funct = {
		//TODO change prototypes when not using dynamically allocated memory!!!!!!
		// a -> z
		{stack_push, "cay2", FAVIM_STACK_PID, FAVIM_STACK_PUSH_FID},
		{stack_pop, "zav1", FAVIM_STACK_PID, FAVIM_STACK_POP_FID},
		{stack_reset, "cav1", FAVIM_STACK_PID, FAVIM_STACK_RESET_FID},
	},
};

/// Main event handling function
static int8_t module(void *state, Message *msg)
{
	stack_state_t *s = (stack_state_t *) state;

	switch (msg->type){
		case MSG_INIT:
			{
				s->pid = msg->did;
				DEBUG("FAViM: Stack initialized\n");
				return SOS_OK;
			}
		case MSG_FINAL:
			{
				return SOS_OK;
			}
		default:
			return -EINVAL;
	}
	return SOS_OK;
}

/// Push a variable onto a FAViM stack
static int8_t stack_push(func_cb_ptr proto, FavimOperandStack* stack, FavimVariable var)
{
	if(stack->sp >= FAVIM_STACK_DEPTH){
		/// TODO raise an error
		return -EINVAL;
	} else {
		stack->stack[(int)stack->sp] = var;
		stack->sp++;
		return FAVIM_OK;
	}
}

/// Pop a variable of a FAViM stack
static FavimVariable* stack_pop(func_cb_ptr proto, FavimOperandStack* stack)
{
	if(stack->sp == 0) {
		///TODO raise an error / implement properly
		return NULL;
	} else {
		stack->sp--;
		return &(stack->stack[(int)stack->sp]);
	}
}

/// Reset a FAViM stack
static int8_t stack_reset(func_cb_ptr proto, FavimOperandStack* stack)
{
	stack->sp=0;
	return FAVIM_OK;
}

#ifndef _MODULE_
mod_header_ptr stack_get_header()
{
	return sos_get_header_address(mod_header);
}
#endif

