/**
 * @file favim.h
 * @brief Variable types in a FAViM virtual machine
 * @author Wouter Horr&eacute;
 */

#ifndef _FAVIM_TYPES_H_
#define _FAVIM_TYPES_H_

#include "favim_constants.h"
#include "sos_constants.h"

// Why doesn't C have booleans?
typedef unsigned char BOOL;
#define FALSE 0
#define TRUE 1

#define FAVIM_BUF_LEN 8

// typedefs
typedef struct FavimDataBuffer FavimDataBuffer;
typedef struct FavimBufferVariable FavimBufferVariable;
typedef struct FavimValueVariable FavimValueVariable;
typedef struct FavimVariable FavimVariable;
typedef uint8_t FavimVariableId;
typedef enum FavimVariableType FavimVariableType;
typedef enum FavimDataType FavimDataType;

//! An opcode in FAViM
typedef uint8_t FavimOpCode;

//! The several variable types
enum FavimVariableType
{
	FAVIM_TYPE_NONE = 0,
	FAVIM_TYPE_BUFFER = 1,
	FAVIM_TYPE_INTEGER = 2,
};

//! The several data types
enum FavimDataType
{
	FAVIM_DATA_NONE = 0,
	FAVIM_DATA_INTEGER = 1,
};

//! A data buffer
struct FavimDataBuffer
{
	FavimDataType type;
	uint8_t size;
	int16_t entries[FAVIM_BUF_LEN];
}  __attribute__((packed));

//! A buffer variable
struct FavimBufferVariable
{
	FavimDataBuffer* var;
};

//! A value variable
struct FavimValueVariable
{
	int16_t var;
}  __attribute__((packed));

//! A variabele in FAViM
struct FavimVariable
{
	FavimVariableType type;
	union 
	{
		FavimValueVariable value;
		FavimBufferVariable buffer;
	};
} __attribute__((packed));


//! Set a chunk of memory to a given value
static inline void *nmemset(void *to, int val, size_t n);
static inline void *nmemset(void *to, int val, size_t n)
{
  char *cto = to;
  while (n--) *cto++ = val;
  return to;
}

//! Copy a chunk of memory to another
static inline void memcopy(char *dest, char *src, uint16_t length);
static inline void memcopy(char *dest, char *src, uint16_t length) 
{
	int i=0;
	for(i=0; i < length; i++)
		*(dest+i) = *(src+i);
	return;
}


#endif

