#ifndef _FAVIM_CONTEXT_H_
#define _FAVIM_CONTEXT_H_

//needed typedefs
typedef struct FavimContext FavimContext;
typedef enum FavimContextState FavimContextState;

#include "../favim.h"
#include "stack.h"
#include "../capsulestore/capsule.h"
#include "vm.h"
#include "../util/favim_queue.h"

//TODO temporary
typedef uint8_t FavimLocalVariable;
#define FAVIM_LOCK_COUNT 2
//TODO

enum FavimContextState {
  FAVIM_CONTEXT_STATE_HALT,
  FAVIM_CONTEXT_STATE_WAITING,
  FAVIM_CONTEXT_STATE_READY,
  FAVIM_CONTEXT_STATE_RUN,
  FAVIM_CONTEXT_STATE_BLOCKED,
};


//! Structure representing a contex in a FavimVM
struct FavimContext
{
	FavimCapsuleIndex pc;									//!< Current pc value
	FavimContextState state;								//!< Context state
	FavimContextId which;									//!< Context ID
	//TODO lock count dynamic !
	uint8_t* useSet;										//!< Held locks (bitmask)
	uint8_t* releaseSet;									//!< Pending lock releases (bitmask)
	uint8_t* acquireSet;									//!< Locks to acquire (bitmask)
	list_link_t link;										//!< Link entry for wait queues
	FavimQueue* queue;										//!< Current wait queue
	uint8_t num_executed;									//!< How long is the context executing
	FavimVM* vm;											//!< The VM to which this context belongs
	FavimVariable** locals;									//!< The local variables of the context
	FavimCapsule* capsule;									//!< The capsule that is executing within this context
	FavimOperandStack* opstack;								//!< The operandstack of the context
} __attribute__((packed));

// TODO waar de grens macro's <-> ft
// TODO static inline versus macro?
// TODO uberhaupt nodig?

//! Return the program counter of the given context
static inline FavimCapsuleIndex favim_context_get_pc(FavimContext * context)
{
	return context->pc;
}

//! Reset the program counter of the given context
static inline void favim_context_reset_pc(FavimContext * context)
{
	context->pc=0;
}

//! Get a local variable with the given id
static inline FavimVariable* favim_context_get_local(FavimContext * context, FavimVariableId id)
{
	return context->locals[(int)id];
}

//! Get the capsule currently executing in this context
static inline FavimCapsule* favim_context_get_capsule(FavimContext * context)
{
	return context->capsule;
}

//! Get the set of currently held locks
static inline uint8_t* favim_context_get_use_set(FavimContext * context)
{
	return context->useSet;
}

//! Get the set of locks to acquire
static inline uint8_t* favim_context_get_acquire_set(FavimContext * context)
{
	return context->acquireSet;
}

//! Get the set of locks to release
static inline uint8_t* favim_context_get_release_set(FavimContext * context)
{
	return context->acquireSet;
}

//! Get the vm wherein the given context is running
static inline FavimVM* favim_context_get_vm(FavimContext * context)
{
	return context->vm;
}

enum {
	FAVIM_CONTEXT_SET_PC_FID = 0,
	FAVIM_CONTEXT_SET_LOCAL_FID = 1,
	FAVIM_CONTEXT_SET_CAPSULE_FID = 2,
	FAVIM_CONTEXT_SET_USE_SET_FID = 3,
	FAVIM_CONTEXT_SET_ACQUIRE_SET_FID = 4,
	FAVIM_CONTEXT_SET_RELEASE_SET_FID = 5,
	FAVIM_CONTEXT_RESET_FID = 6,
};


//TODO some kind of init
//typedefs needed to call the exported functions
typedef void (*favim_context_set_pc_proto)(func_cb_ptr, FavimContext*, FavimCapsuleIndex);
//TODO NODIG ??? get -> var aanpassen rechtstreeks?
typedef void (*favim_context_set_local_proto)(func_cb_ptr, FavimContext*, FavimVariableId, FavimVariable*);
typedef void (*favim_context_set_capsule_proto)(func_cb_ptr, FavimContext*, FavimCapsule*);
//op niveau individuele lock?
typedef void (*favim_context_set_use_set_proto)(func_cb_ptr, FavimContext*, uint8_t*);
typedef void (*favim_context_set_acquire_set_proto)(func_cb_ptr, FavimContext*, uint8_t*);
typedef void (*favim_context_set_release_set_proto)(func_cb_ptr, FavimContext*, uint8_t*);
//
typedef void (*favim_context_reset_proto)(func_cb_ptr, FavimContext*);


#endif
