#include <util/favim_queue.h>

int8_t queue_init(FavimQueue* queue) 
{
	DEBUG("FAVIM: Initializing queue %p\n", queue);
	list_init(&queue->queue);
	return SOS_OK;
}

uint8_t queue_empty(FavimQueue* queue) 
{
	bool emp = list_empty(&queue->queue);
	DEBUG("FAVIM: Testing if queue at %p is empty: %s.\n", queue, (emp)? "true":"false");
	return emp;
}

int8_t queue_enqueue(FavimContext* context, FavimQueue* queue, FavimContext* element) 
{
	DEBUG("FAVIM (%i): Enqueue %i on %p...", (int)context->which, (int)element->which, queue);
	if (element->queue) 
	{
		//error_error(error_proto, context, MATE_ERROR_QUEUE_ENQUEUE);
		DEBUG("FAILURE, already there\n");
		return -EINVAL;
	}
	element->queue = queue;
	list_insert_head(&queue->queue, &element->link);
	DEBUG("success\n"); 
	return SOS_OK;
}


FavimContext* queue_dequeue(FavimContext* context, FavimQueue* queue) 
{
	FavimContext* rval;
	list_link_t* listLink;;

	if (list_empty(&queue->queue)) 
	{
		//error_error(error_proto, context, MATE_ERROR_QUEUE_DEQUEUE);
		return NULL;
	}
    
	listLink = queue->queue.l_prev;
	rval = (FavimContext*)((char*)listLink - offsetof(FavimContext, link));
	list_remove(listLink);
	rval->link.l_next = 0;
	rval->link.l_prev = 0;
	rval->queue = NULL;
	if (rval != NULL) 
	{
		DEBUG("FAVIM: Dequeuing context %i from queue %p.\n", (int)rval->which, queue);
	}
	return rval;
}

int8_t queue_remove(FavimContext* context, FavimQueue* queue, FavimContext* element) 
{
	DEBUG("FAVIM (%i): Removing context %i from queue %p.\n", (int)context->which, (int)element->which, queue);
	if (element->queue != queue) 
	{
		return -EINVAL;
	}
	element->queue = NULL;
	if (!(element->link.l_next && element->link.l_prev)) 
	{
		return -EINVAL;
	}
	else 
	{
		list_remove(&element->link);
		return SOS_OK;
	}
}
