#ifndef _FAVIM_VM_H_
#define _FAVIM_VM_H_

// needed typedefs
typedef uint8_t FavimContextId;
typedef struct FavimVM FavimVM;
typedef struct FavimLock FavimLock;

#include "../favim.h"
#include "context.h"
#include "../operationstore/library_manager.h"

#define OPCODE_BITS 5
#define LIBRARY_MASK 0x07
#define OPCODE_MASK 0x1F

#define REBOOT_CONTEXT_ID 0
#define TIMER0_CONTEXT_ID 1

struct FavimLock{
  FavimContext* holder;
}  __attribute__((packed));


//! Structure representing a VM in FAViM
struct FavimVM
{
	FavimContext** contexts;		//! The contexts running in this vm
	FavimVariable* globals[FAVIM_GLOBAL_COUNT];		//! The global variables of the VM
	FavimLock locks[FAVIM_GLOBAL_COUNT];			//! Which globals are locked?
} __attribute__((packed));

//! Get a global variable from the given vm	
//! @pre id must be within the range of globals
static inline FavimVariable* favim_vm_get_global(FavimVM * vm, FavimVariableId id)
{
	return vm->globals[(int)id];
}

//! Lock a global variable of the given vm
//! @pre id must be within the range of globals
static inline void favim_vm_lock_global(FavimVM * vm, FavimVariableId id)
{
	//TODO implementation
}

//! Unlock a global variable of the given vm
//! @pre id must be within the range of globals
static inline void favim_vm_unlock_global(FavimVM * vm, FavimVariableId id)
{
	//TODO implementation
}

//! Get the context with the given id
//! @pre id must be in the range of contexts
static inline FavimContext* favim_vm_get_context(FavimVM * vm, FavimContextId id)
{
	return vm->contexts[(int)id];
}

//TODO setGlobal ????

enum
{
	FAVIM_VM_GET_LIBRARY_FID,
	FAVIM_VM_GET_OPCODE_FID,
	FAVIM_VM_GET_NEEDED_LIBS_FID,
	FAVIM_VM_LOCK_GLOBAL_FID,
	FAVIM_VM_UNLOCK_GLOBAL_FID,
	FAVIM_VM_IS_LOCKED_FID,
};

typedef FavimLibraryId (*favim_vm_get_library_proto)(func_cb_ptr,FavimOpCode);
typedef FavimLibraryId (*favim_vm_get_opcode_proto)(func_cb_ptr,FavimOpCode);
typedef FavimLibraryBitmask (*favim_vm_get_needed_libs_proto)(func_cb_ptr, FavimOpCode);
typedef int8_t (*favim_vm_lock_global_proto)(func_cb_ptr, FavimContext*, FavimVariableId);
typedef int8_t (*favim_vm_unlock_global_proto)(func_cb_ptr,FavimContext*, FavimVariableId);
typedef BOOL (*favim_vm_is_locked_proto)(func_cb_ptr, FavimVariableId);
// method for libbitmask in capsule to global libmask

#endif
