/**
 * @file library.c
 * @brief The interface to the libraries present in the system
 * @author Wouter Horr&eacute;
 */
 
#include <module.h>

#include <operationstore/library.h>

//! The state of the library interface
typedef struct {
	sos_pid_t pid;
} library_state_t;

// Forward declarations
static int8_t module(void *state, Message *e);
static uint8_t bytelength(func_cb_ptr, FavimLibraryId lib, FavimOpCode opcode);
static uint8_t* uses_globals(func_cb_ptr, FavimLibraryId lib, FavimOpCode opcode);


//! The module header of the library interface module
static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = FAVIM_LIB_PID,
	.state_size     = sizeof(library_state_t),
    .num_timers     = 0,
	.num_sub_func   = 0,
	.num_prov_func  = 0,
	.module_handler = module,
	.funct = {
		{bytelength, "CCC2", FAVIM_LIB_PID, FAVIM_LIB_BYTELENGTH_FID},
		{uses_globals, "ECC2", FAVIM_LIB_PID, FAVIM_LIB_USES_GLOBALS_FID},
	},
};

//! The main message handling function
static int8_t module(void *state, Message *msg)
{
    library_state_t *s = (library_state_t *) state;

    switch (msg->type){

        case MSG_INIT:
		{
			s->pid = msg->did;
			return SOS_OK;
		}


        case MSG_FINAL:
        {
			return SOS_OK;
        }
        
        case FAVIM_CMD_LIB_EXECUTE:
        {
        	//TODO implementation
        	return SOS_OK;
        }

        default:
			return -EINVAL;
	}

    return SOS_OK;
}

//! Get the bytelength of the given opcode
static uint8_t bytelength(func_cb_ptr proto, FavimLibraryId lib, FavimOpCode opcode)
{
	//TODO implementation
	return 0;
}

//! Returns the globals used by the given opcode (bitmask)
static uint8_t* uses_globals(func_cb_ptr proto, FavimLibraryId lib, FavimOpCode opcode)
{
	//TODO implementation
	return NULL;
}


#ifndef _MODULE_
mod_header_ptr library_get_header()
{
    return sos_get_header_address(mod_header);
}
#endif

