#ifndef _FAVIM_VM_H_
#define _FAVIM_VM_H_

// needed typedefs
typedef uint8_t FavimContextId;
typedef struct FavimVM FavimVM;
typedef struct FavimLock FavimLock;

#include "../favim.h"
#include "context.h"
#include "../operationstore/library_manager.h"

#define OPCODE_BITS 5
#define LIBRARY_MASK 0x07
#define OPCODE_MASK 0x1F

#define REBOOT_CONTEXT_ID 0
#define TIMER0_CONTEXT_ID 1

struct FavimLock{
  FavimContext* holder;
}  __attribute__((packed));


//! Structure representing a VM in FAViM
struct FavimVM
{
	FavimContext** contexts;		//! The contexts running in this vm
	FavimVariable* globals[FAVIM_GLOBAL_COUNT];		//! The global variables of the VM
	FavimLock locks[FAVIM_GLOBAL_COUNT];			//! Which globals are locked?
} __attribute__((packed));

enum
{
	FAVIM_VM_GET_LIBRARY_FID,
	FAVIM_VM_GET_OPCODE_FID,
	FAVIM_VM_GET_NEEDED_LIBS_FID,
	FAVIM_VM_LOCK_GLOBAL_FID,
	FAVIM_VM_UNLOCK_GLOBAL_FID,
	FAVIM_VM_IS_LOCKED_FID,
	FAVIM_VM_HALT_FID,
};

typedef FavimLibraryId (*favim_vm_get_library_proto)(func_cb_ptr,FavimOpCode);
typedef FavimLibraryId (*favim_vm_get_opcode_proto)(func_cb_ptr,FavimOpCode);
typedef FavimLibraryBitmask (*favim_vm_get_needed_libs_proto)(func_cb_ptr, FavimOpCode);
typedef int8_t (*favim_vm_lock_global_proto)(func_cb_ptr, FavimContext*, FavimVariableId);
typedef int8_t (*favim_vm_unlock_global_proto)(func_cb_ptr,FavimContext*, FavimVariableId);
typedef BOOL (*favim_vm_is_locked_proto)(func_cb_ptr, FavimVariableId);
typedef void (*favim_vm_halt_proto)(func_cb_ptr);

#endif
