/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */

#include <util/davim_queue.h>

int8_t queue_init(DavimQueue* queue) 
{
	DEBUG("DAVIM: Initializing queue %p\n", queue);
	list_init(&queue->queue);
	return SOS_OK;
}

uint8_t queue_empty(DavimQueue* queue) 
{
	bool emp = list_empty(&queue->queue);
	DEBUG("DAVIM: Testing if queue at %p is empty: %s.\n", queue, (emp)? "true":"false");
	return emp;
}

int8_t queue_enqueue(DavimContext* context, DavimQueue* queue, DavimContext* element) 
{
	DEBUG("DAVIM (VM %i / CON %i): Enqueue %i on %p...\n", (int)context->vm, (int)context->which, (int)element->which, queue);
	if (element->queue) 
	{
		DEBUG("FAILURE, already there\n");
		return -EINVAL;
	}
	element->queue = queue;
	list_insert_head(&queue->queue, &element->link);
	DEBUG("success\n"); 
	return SOS_OK;
}


DavimContext* queue_dequeue(DavimContext* context, DavimQueue* queue) 
{
	DavimContext* rval;
	list_link_t* listLink;;

	if (list_empty(&queue->queue)) 
	{
		//error_error(error_proto, context, MATE_ERROR_QUEUE_DEQUEUE);
		return NULL;
	}
    
	listLink = queue->queue.l_prev;
	rval = (DavimContext*)((char*)listLink - offsetof(DavimContext, link));
	list_remove(listLink);
	rval->link.l_next = 0;
	rval->link.l_prev = 0;
	rval->queue = NULL;
	if (rval != NULL) 
	{
		DEBUG("DAVIM: Dequeuing context %i from queue %p.\n", (int)rval->which, queue);
	}
	return rval;
}

int8_t queue_remove(DavimContext* context, DavimQueue* queue, DavimContext* element) 
{
	DEBUG("DAVIM (%i): Removing context %i from queue %p.\n", (int)context->which, (int)element->which, queue);
	if (element->queue != queue) 
	{
		return -EINVAL;
	}
	element->queue = NULL;
	if (!(element->link.l_next && element->link.l_prev)) 
	{
		return -EINVAL;
	}
	else 
	{
		list_remove(&element->link);
		return SOS_OK;
	}
}
