/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
/**
 * @file capsule.h
 * @brief The Interface of a DAViM capsule
 * @author Wouter Horr&eacute;
 */

#ifndef _DAVIM_CAPSULE_H_
#define _DAVIM_CAPSULE_H_

//needed typedefs
typedef struct DavimCapsule DavimCapsule;

#include "../davim.h"

// maximum length of a capsule: 256 bytes
#define DAVIM_CAPSULE_MAX_SIZE 256

//typedef uint16_t DavimCapsuleOption;
//! The index of a davim capsule
typedef uint8_t DavimCapsuleIndex;
//! The length of a davim capsule
typedef DavimCapsuleIndex DavimCapsuleLength;
//! The version of a davim capsule
typedef uint32_t DavimCapsuleVersion;

//! A capsule in DAViM
struct DavimCapsule
{
	//DavimCapsuleOption options;
	DavimCapsuleVersion version;
	uint8_t usedVars[(DAVIM_GLOBAL_COUNT + 7) / 8];
	DavimCapsuleLength dataSize;
//	DavimOpCode data[DAVIM_CAPSULE_SIZE];
	DavimOpCode *data;
} __attribute__((packed));

enum
{
	DAVIM_CAPSULE_NEW_FID,
	DAVIM_CAPSULE_FINALIZE_FID,
};

// Typedefs needed for calling the exported functions
typedef DavimCapsule* (*davim_capsule_new_proto)(func_cb_ptr);
typedef void (*davim_capsule_finalize_proto)(func_cb_ptr, DavimCapsule*);

//! The state of the capsule module
typedef struct {
	sos_pid_t pid;
	DavimOpCode default_capsule[2];
} capsule_state_t;


//! The default capsule
//extern DavimOpCode default_capsule[];

#endif

