/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
/**
 * @file capsule_manager.h
 * @brief The interface of the DAViM Capsule Manager
 * @author Wouter Horr&eacute;
 */

#ifndef _DAVIM_CAPSULE_MANAGER_H_
#define _DAVIM_CAPSULE_MANAGER_H_

typedef struct DavimCapsuleMsg DavimCapsuleMsg;
typedef struct DavimCapsuleIdVersionPair DavimCapsuleIdVersionPair;

#include "../davim.h"
#include "capsule.h"
#include "../vmstore/vm_manager.h"

//! A Capsule ID
typedef uint8_t DavimCapsuleId;

struct DavimCapsuleMsg
{
	DavimCapsuleId id;
	DavimCapsule capsule;
} __attribute__((packed)) ;

struct DavimCapsuleIdVersionPair
{
	DavimCapsuleId id;
	DavimCapsuleVersion version;
} __attribute__((packed)) ;

//! The function id of the exported functions
enum
{
	DAVIM_CAPSULE_MANAGER_ADD_CAPSULE_FID,
	DAVIM_CAPSULE_MANAGER_REMOVE_CAPSULE_FID,
	DAVIM_CAPSULE_MANAGER_GET_CAPSULE_FID,
	DAVIM_CAPSULE_MANAGER_GET_NB_CAPSULES_FID,
	DAVIM_CAPSULE_MANAGER_GET_ALL_VERSIONS_FID,
};

// Typedefs needed to call the exported functions
typedef int8_t (*davim_capsule_manager_add_capsule_proto)(func_cb_ptr, DavimVMId, DavimContextId, DavimCapsule*);
typedef int8_t (*davim_capsule_manager_remove_capsule_proto)(func_cb_ptr, DavimVMId, DavimContextId);
typedef DavimCapsule * (*davim_capsule_manager_get_capsule_proto)(func_cb_ptr, DavimCapsuleId);
typedef uint8_t (*davim_capsule_manager_get_nb_capsules_proto)(func_cb_ptr);
typedef DavimCapsuleIdVersionPair* (*davim_capsule_manager_get_all_versions_proto)(func_cb_ptr);

typedef struct HashEntry HashEntry;

struct HashEntry {
	DavimCapsuleId id;
	DavimCapsule* capsule;
	HashEntry* next;
};

//! The state of the capsule manager
typedef struct {
	func_cb_ptr vm_halt;
	func_cb_ptr analyse_capsule;
	sos_pid_t pid;
	uint8_t nb_capsules;
	HashEntry * capsules[8];
} capsule_manager_state_t;



#endif
