/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
/**
 * @mainpage DAViM - Dynamisch Aanpasbare Virtuele Machines
 * @anchor mainpage
 * 
 * @section pre Precondities
 * 
 * DAViM maakt gebruikt van <a href="http://nesl.ee.ucla.edu/projects/sos/">Sensor Operating System (SOS)</a>.
 * De uitleg op deze pagina gaat er van uit dat de <b>CVS-versie</b> van SOS reeds is ge&iuml;nstalleerd. Ook de AVR toolchain moet reeds ge&iuml;nstalleerd zijn, zie
 * <a href="http://nesl.ee.ucla.edu/projects/sos/tutorial/installation.html">http://nesl.ee.ucla.edu/projects/sos/tutorial/installation.html</a>.
 * 
 * @section install Installatie
 * 
 * De tools die deel uit maken van DAViM moeten gecompileerd worden:
 * <pre>
 * $ cd /path/to/davim/src/tools/
 * $ make
 * </pre>
 * 
 * Om makkelijk te kunnen werken voeg je de tools directory best toe aan je PATH. 
 * Dit doe je in het bestand ~/.bashrc (of het equivalent indien je andere shell gebruikt):
 * <pre>
 * export PATH=$PATH:/path/to/davim/src/tools/
 * </pre>
 * 
 * DAViM moet ook weten waar SOS zich bevindt op je systeem, zet daarom de volgende omgevingsvariabele:
 * <pre>
 * export SOS1XDIR=/path/to/sos-1.x
 * </pre>
 *
 * Er moet ook een constante in een header van SOS worden aangepast, namelijk
 * in platform/micaz/include/codemem_conf.h moet de volgende wijziging
 * aangebracht worden:
 * <pre>
 *  //  CODEMEM_START_PAGE = 224L,
 *      CODEMEM_START_PAGE = 400L,
 * </pre>			
 * 
 * @section example Voorbeeld
 * 
 * @see src/configs/voorbeeld/
 * @see src/configs/voorbeeld_base/
 * 
 * DAViM bevat een voorbeeld die het gebruik van het systeem illustreert. Dit voorbeeld bestaat uit
 * twee configuraties: voorbeeld en voorbeeld_base. De laatste is bedoeld voor de knoop die met de computer is verbonden en zo
 * de brug vormt tussen de gebruiker en het sensornetwerk.
 * 
 * @subsection vb_sim Simulatie
 * 
 * We behandelen het voorbeeld eerst in simulatie. De beide configuraties moeten gecompileerd worden voor het 'sim' target van SOS:
 * <pre>
 * $ cd /path/to/davim/src/configs/voorbeeld/
 * $ make sim EMU_PLATFORM=mica2
 * $ cd /path/to/davim/src/configs/voorbeeld_base/
 * $ make sim EMU_PLATFORM=mica2
 * </pre>
 * 
 * We starten eerst de basisknoop:
 * <pre>
 * $ cd /path/to/davim/src/configs/voorbeeld_base/
 * $ ./voorbeeld_base.exe -n 0 -f ../topo.def -s 2390
 * </pre>
 * 
 * En SOS-server:
 * <pre>
 * $ sossrv.exe -n 127.0.0.1:2390
 * </pre>
 * 
 * Vervolgens starten we de rest van de knopen. Dit doen we als volgt (de knoopidentificatie na de parameter -n moet voor elke knoop anders zijn):
 * <pre>
 * $ cd /path/to/davim/src/configs/voorbeeld/
 * $ ./voorbeeld.exe -n 1 -f ../topo.def
 * </pre>
 * 
 * Daarna injecteren we de operatiebibliotheek voor het versturen van een bericht in het sensornetwerk. De code voor operatiebibliotheek is in werkelijkheid
 * reeds in de applicatie meegecompileerd. Het laden van de module moet echter nog gebeuren en het verspreiden ervan door het netwerk wordt gesimuleerd. Om de
 * te kunnen injecteren heeft de SOS-tool die hiervoor dient informatie nodig over de module. Die haalt hij uit de .sos waarnaar de module gecompileerd wordt.
 * We kunnen een module echter niet compileren voor het sim target, dus compileren we voor het micaz platform:
 * <pre>
 * $ cd /path/to/davim/src/libs/netlib/
 * $ make micaz
 * $ sos_insmod.sh netlib.sos
 * </pre>
 * 
 * Vervolgens moeten we de standaard virtuele machine op de knopen uitbreiden met een extra context en de verwijzingen aanpassen om van de netlib bibliotheek
 * gebruik te maken.
 * <pre>
 * $ cd /path/to/davim/src/configs/voorbeeld/
 * $ davim-vmgen voorbeeldvm.txt voorbeeldvm.bin
 * $ davim-injector --vm voorbeeldvm.bin 0
 * </pre>
 * 
 * Tenslotte zetten we de twee scriptjes van de voorbeeld applicatie om in bytecode en injecteren ze in het netwerk:
 * <pre>
 * $ cd /path/to/davim/src/configs/voorbeeld/
 * $ davim-as meetenzend.txt meetenzend.bc
 * $ davim-injector --capsule meetenzend.bc 0 1
 * $ davim-as zettimer.txt zettimer.bc
 * $ davim-injector --capsule zettimer.bc 0 0
 * </pre>
 * 
 * Na een tijdje zien we op geregelde tijdstippen een meting van de lichtsensor binnenkomen via SOS-server:
 * <pre>
 * --- Received from Serial, CRC OK! ---
 * Dest Mod Id: 0x83
 * Src  Mod Id: 0x83
 * Dest Addr  : 0xFFFF
 * Src  Addr  : 0x01
 * Msg Type   : 32
 * Msg Length : 2
 * Msg Data:
 * 0xBD 0x0A
 * CRC: 0xFBEF
 * </pre>
 * 
 * @subsection MICAz-sensorknopen
 * 
 * De voorbeeld applicatie gebruiken op MICAz-sensorknopen is heel gelijkaardig als bij simulatie. We kunnen echter de module met de operatiebibliotheek
 * niet laden omdat er te weinig geheugen vrij is voor de lader (zie Thesistekst).
 * 
 * De configuraties moeten opnieuw gecompileerd worden en moeten op de sensorknopen geladen worden. We bevestigen een knoop op het programmeerbordje, waarna
 * we de configuraties compileren en laden:
 * <pre>
 * Voor de gewone knopen (ADDRESS vari&euml;ren):
 * $ cd /path/to/davim/src/configs/voorbeeld/
 * $ make micaz install PROG=mib510 PORT=/dev/ttyS0 SOS_GROUP=1 ADDRESS=1
 * 
 * Voor de basisknoop:
 * $ cd /path/to/davim/src/configs/voorbeeld_base/
 * $ make micaz install PROG=mib510 PORT=/dev/ttyS0 SOS_GROUP=1 ADDRESS=0
 * </pre>
 * 
 * Voer nu de stappen uit zoals bij simulatie, echter enkel vanaf het wijzigen van de virtuele machine. De netlib bibliotheek is reeds
 * in de configuratie meegecompileerd omdat het laden onmogelijk is (zie Thesistekst).
 * 
 */
 



/**
 * @file davim.h
 * @brief Variable types in a DAViM virtual machine
 * @author Wouter Horr&eacute;
 */

#ifndef _DAVIM_H_
#define _DAVIM_H_

#include <led.h>
#include "davim_constants.h"
#include "sos_constants.h"

// Why doesn't C have booleans?
typedef unsigned char BOOL;
#define FALSE 0
#define TRUE 1

#define DAVIM_BUF_LEN 8
#define DAVIM_GLOBAL_COUNT 2

#define DAVIM_VM_COUNT 8
#define DAVIM_VM_ID_BITS 3
#define DAVIM_CONTEXT_COUNT 32
#define DAVIM_CONTEXT_ID_BITS 5

// typedefs
typedef struct DavimDataBuffer DavimDataBuffer;
typedef struct DavimBufferVariable DavimBufferVariable;
typedef struct DavimValueVariable DavimValueVariable;
typedef struct DavimVariable DavimVariable;
typedef uint8_t DavimVariableId;
typedef enum DavimVariableType DavimVariableType;
typedef enum DavimDataType DavimDataType;

//! An opcode in DAViM
typedef uint8_t DavimOpCode;

//! The global events in DAVIM
enum
{
	DAVIM_EVENT_GLOBAL_VM_HALTED=1, //This can't be 0 -> comparison always true when comparing id >= ...
	DAVIM_EVENT_GLOBAL_ADD_LIB = DAVIM_EVENT_GLOBAL_VM_HALTED + DAVIM_VM_COUNT,
	DAVIM_EVENT_GLOBAL_REMOVE_LIB,
};

//! The several variable types
enum DavimVariableType
{
	DAVIM_TYPE_NONE = 0,
	DAVIM_TYPE_BUFFER = 1,
	DAVIM_TYPE_INTEGER = 2,
};

//! The several data types
enum DavimDataType
{
	DAVIM_DATA_NONE = 0,
	DAVIM_DATA_INTEGER = 1,
};

//! A data buffer
struct DavimDataBuffer
{
	DavimDataType type;
	uint8_t size;
	int16_t entries[DAVIM_BUF_LEN];
}  __attribute__((packed));

//! A buffer variable
struct DavimBufferVariable
{
	DavimDataBuffer* var;
};

//! A value variable
struct DavimValueVariable
{
	int16_t var;
}  __attribute__((packed));

//! A variabele in DAViM
struct DavimVariable
{
	DavimVariableType type;
	union 
	{
		DavimValueVariable value;
		DavimBufferVariable buffer;
	};
} __attribute__((packed));


//! Set a chunk of memory to a given value
static inline void *nmemset(void *to, int val, size_t n);
static inline void *nmemset(void *to, int val, size_t n)
{
  char *cto = to;
  while (n--) *cto++ = val;
  return to;
}

//! Copy a chunk of memory to another
static inline void memcopy(char *dest, char *src, uint16_t length);
static inline void memcopy(char *dest, char *src, uint16_t length) 
{
	int i=0;
	for(i=0; i < length; i++)
		*(dest+i) = *(src+i);
	return;
}


//! Check the type of a DavimVariable
static inline bool type_check(DavimVariable * var, DavimVariableType type)
{
	return var->type==type;
}

#ifdef PC_PLATFORM
void * my_malloc(uint16_t size, sos_pid_t id);
#else
#define my_malloc(a,b) ker_malloc(a,b)
#endif

static inline void panic(){
#ifdef PC_PLATFORM
	DEBUG("*************************************************************\n");
	DEBUG("DAVIM PANIC *************************************************\n");
	DEBUG("*************************************************************\n");
#else
//	ker_led(LED_GREEN_OFF);
//	ker_led(LED_RED_OFF);
//	ker_led(LED_YELLOW_OFF);
//	ker_led(LED_GREEN_ON);
//	ker_led(LED_RED_ON);
//	ker_led(LED_YELLOW_ON);
//	ker_led(LED_YELLOW_OFF);
//	ker_led(LED_RED_OFF);
//	ker_led(LED_GREEN_OFF);
#endif
}

#endif

