/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
#ifndef _DAVIM_EVENT_MANAGER_H_
#define _DAVIM_EVENT_MANAGER_H_

#include "../davim.h"
#include "../vmstore/vm_manager.h"
#include <sos_timer.h>

typedef uint8_t DavimEventId;
typedef uint8_t DavimTimerInterval;
typedef enum DavimEventType DavimEventType;
typedef enum DavimTimerType DavimTimerType;
typedef struct DavimEventTimer DavimEventTimer;

//! The types of events
enum DavimEventType
{
	DAVIM_EVENT_SINGLE_VM,
	DAVIM_EVENT_GLOBAL,
};

//! The types of timers
enum DavimTimerType
{
	DAVIM_TIMER_REPEAT = TIMER_REPEAT,
	DAVIM_TIMER_ONE_SHOT = TIMER_ONE_SHOT,
};

//! The function ids of the exported functions
enum
{
	DAVIM_EVENT_MANAGER_START_TIMER_FID,
	DAVIM_EVENT_MANAGER_STOP_TIMER_FID,
	DAVIM_EVENT_MANAGER_INIT_EVENT_FID,
	DAVIM_EVENT_MANAGER_TRIGGER_EVENT_FID,
	DAVIM_EVENT_MANAGER_SIGNAL_HALT_FID,
};	

//! Structure representing a davim timer
struct DavimEventTimer {
	DavimEventType type;
	DavimEventId id;
	DavimValueVariable interval;
	DavimValueVariable ticks;
	DavimEventTimer* next;
}__attribute__((packed));

typedef int8_t (*davim_event_manager_start_timer_proto)(func_cb_ptr,DavimEventType,DavimVMId,DavimEventId,DavimTimerType,DavimTimerInterval);
typedef int8_t (*davim_event_manager_stop_timer_proto)(func_cb_ptr,DavimEventType,DavimVMId,DavimEventId);
//typedef int8_t (*davim_event_manager_init_event_proto)(func_cb_ptr,DavimEventId);
typedef int8_t (*davim_event_manager_trigger_event_proto)(func_cb_ptr,DavimEventType,DavimVMId,DavimEventId);


//! The state of the DAViM event manager
typedef struct {
	sos_pid_t pid;
	DavimEventTimer* timerlist;
//	uint8_t timercount;
} event_man_state_t;


#endif
