/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
#ifndef _DAVIM_SOS_CONSTANTS_H_
#define _DAVIM_SOS_CONSTANTS_H_

#include <pid.h>

/**
 * @brief Minimum pid of core davim modules
 * 
 * Equal to APP_MOD_MIN_PID + 64 (192)
 */
#define DAVIM_CORE_MIN_PID (APP_MOD_MIN_PID + 64)
//! Maximum pid of core davim modules
#define DAVIM_CORE_MAX_PID (DAVIM_CORE_MIN_PID + 15)
/**
 * @brief Minimum pid of davim libraries
 * 
 * Equal to APP_MOD_MIN_PID (128)
 * We align them at 128 to make it easy to use masks
 */
#define DAVIM_LIB_MIN_PID APP_MOD_MIN_PID
/**
 * @brief Maximum pid of davim libraries
 * 
 * We limit the number of libraries to 64.
 */
#define DAVIM_LIB_MAX_PID (DAVIM_LIB_MIN_PID + 63)

//! PID's of core davim modules
enum 
{
	DAVIM_VIRUS_PID = DAVIM_CORE_MIN_PID,	//192
	DAVIM_STACK_PID,							//193
	DAVIM_CAPSULE_PID,						//194
	DAVIM_CAPSULE_MANAGER_PID,				//195
	DAVIM_CONTEXT_PID,						//196
	DAVIM_VM_PID,							//197
	DAVIM_CONC_PID,							//198
	DAVIM_LIB_PID,							//199
	DAVIM_LIB_MANAGER_PID,					//200
	DAVIM_SCHEDULER_PID,						//201
	DAVIM_EVENT_MANAGER_PID,					//202
	DAVIM_VM_MANAGER_PID,					//203
};

//! PID's of the libs
enum
{
	DAVIM_LIB_BASICLIB_PID = DAVIM_LIB_MIN_PID, //128
	DAVIM_LIB_TESTLIB_PID,						//129
	DAVIM_LIB_NETLIB_PID = DAVIM_LIB_MIN_PID + 3, //force to 131 for demonstation purposes
};

//! The start of the command ids in davim
#define DAVIM_CMD_START MOD_CMD_START
//! The start of the message ids in davim
#define DAVIM_MSG_START MOD_MSG_START

//! The command messages in davim
enum
{
//	DAVIM_CMD_VM_ADD_CONTEXT = DAVIM_CMD_START,
//	DAVIM_CMD_VM_REMOVE_CONTEXT,
	DAVIM_CMD_VM_INSTALL_DESCRIPTION = DAVIM_CMD_START,
	DAVIM_CMD_VM_EVENT_OCCURED,
	DAVIM_CMD_VM_REBOOT,
	DAVIM_CMD_LIB_EXECUTE,
	DAVIM_CMD_LIB_MANAGER_ADD_LIBRARY,
	DAVIM_CMD_LIB_MANAGER_REMOVE_LIBRARY,
	DAVIM_CMD_VIRUS_CAPSULE_VERSION_TIMER_TASK,
	DAVIM_CMD_VIRUS_CAPSULE_TIMER_TASK,
	DAVIM_CMD_VIRUS_CAPSULE_CHECK_NEED_TASK,
	DAVIM_CMD_VIRUS_VM_VERSION_TIMER_TASK,
	DAVIM_CMD_VIRUS_VM_TIMER_TASK,
	DAVIM_CMD_SCHEDULER_HALT,
	DAVIM_CMD_SCHEDULER_RESUME,
	DAVIM_CMD_SCHEDULER_RUN_TASK,
	DAVIM_CMD_CAPS_MANAGER_CAPS_ARRIVED,
//	DAVIM_CMD_VM_MAN_ADD_VM,
//	DAVIM_CMD_VM_MAN_REMOVE_VM,
	DAVIM_CMD_VM_MAN_VMMSG_ARRIVED,
	DAVIM_CMD_TRIGGER_GLOBAL,
};

//! The messages in davim
enum
{
	DAVIM_MSG_VIRUS_CAPSULECHUNKMSG = DAVIM_MSG_START,
	DAVIM_MSG_VIRUS_CAPSULESTATUSMSG,
	DAVIM_MSG_VIRUS_CAPSULE_VERSIONMSG,
//	DAVIM_MSG_VIRUS_CAPSULE_VERSIONREQUESTMSG,
	DAVIM_MSG_VIRUS_VMMSG,
	DAVIM_MSG_VIRUS_VM_VERSIONMSG,
};

//! The timers in davim
enum
{
//	SUBSCRIBE_TIMER = 0,
//	ERROR_TIMER = 1,
	DAVIM_VIRUS_CAPSULE_VERSION_TIMER_ID,
	DAVIM_VIRUS_CAPSULE_TIMER_ID,
	DAVIM_VIRUS_VM_VERSION_TIMER_ID,
	DAVIM_VIRUS_VM_TIMER_ID,
	DAVIM_EVENT_TIMER_ID,
//	CLOCK_TIMER = 4,
};


#endif

