/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
#ifndef _DAVIM_CONTEXT_H_
#define _DAVIM_CONTEXT_H_

//needed typedefs
typedef struct DavimContext DavimContext;
typedef enum DavimContextState DavimContextState;

#include "../davim.h"
#include "stack.h"
#include "../capsulestore/capsule.h"
#include "vm.h"
#include "vm_manager.h"
#include "../util/davim_queue.h"

#define DAVIM_LOCAL_COUNT 2

enum DavimContextState {
  DAVIM_CONTEXT_STATE_HALT,
  DAVIM_CONTEXT_STATE_WAITING,
  DAVIM_CONTEXT_STATE_READY,
  DAVIM_CONTEXT_STATE_RUN,
  DAVIM_CONTEXT_STATE_BLOCKED,
  DAVIM_CONTEXT_STATE_UNBLOCKED,
};


//! Structure representing a context in a DavimVM
struct DavimContext
{
	DavimVMId vm;											//!< The VM to which this context belongs
	DavimContextId which;									//!< Context ID
	DavimCapsuleIndex pc;									//!< Current pc value
	DavimContextState state;									//!< Context state
	uint8_t num_executed;									//!< How long is the context executing
	void* locals[DAVIM_LOCAL_COUNT];							//!< Entry points for the local variables of the context
	DavimCapsule* capsule;									//!< The capsule that is executing within this context
	DavimOperandStack* opstack;								//!< The operandstack of the context
	uint8_t heldSet[(DAVIM_GLOBAL_COUNT + 7) / 8];			//!< Held locks (bitmask)
	uint8_t releaseSet[(DAVIM_GLOBAL_COUNT + 7) / 8];		//!< Pending lock releases (bitmask)
	uint8_t acquireSet[(DAVIM_GLOBAL_COUNT + 7) / 8];		//!< Locks to acquire (bitmask)
	list_link_t link;										//!< Link entry for wait queues
	DavimQueue* queue;										//!< Current wait queue
} __attribute__((packed));


enum
{
	DAVIM_CONTEXT_NEW_FID,
	DAVIM_CONTEXT_FINALIZE_FID,
	DAVIM_CONTEXT_RESET_FID,
};

typedef DavimContext* (*davim_context_new_proto)(func_cb_ptr, DavimVMId, DavimContextId);
typedef void (*davim_context_finalize_proto)(func_cb_ptr, DavimContext*);
typedef void (*davim_context_reset_proto)(func_cb_ptr, DavimContext*);

//! The state of the context module
typedef struct {
	func_cb_ptr stack_new;
	func_cb_ptr stack_finalize;
	func_cb_ptr stack_reset;
	func_cb_ptr capsule_new;
	func_cb_ptr capsule_finalize;
	func_cb_ptr add_capsule;
	func_cb_ptr remove_capsule;
	sos_pid_t pid;
} context_state_t;

#endif
