/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
/**
 * @file stack.h
 * @brief The DAViM stack interface
 * @author Wouter Horr&eacute;
 */

#ifndef _DAVIM_STACK_H_
#define _DAVIM_STACK_H_

//needed typedefs
typedef struct DavimOperandStack DavimOperandStack;

#include "../davim.h"
#include "../davim_constants.h"

#define DAVIM_STACK_DEPTH 2

//! The function ids of the exported functions
enum 
{
	DAVIM_STACK_NEW_FID,
	DAVIM_STACK_FINALIZE_FID,
	DAVIM_STACK_PUSH_FID,
	DAVIM_STACK_POP_FID,
	DAVIM_STACK_RESET_FID,
};

//! A DAViM operand stack
struct DavimOperandStack
{
	uint8_t sp;
	void* stack[DAVIM_STACK_DEPTH];
} __attribute__((packed));

// Typedefs needed to call the functions
typedef DavimOperandStack* (*davim_stack_new_proto)(func_cb_ptr);
typedef void (*davim_stack_finalize_proto)(func_cb_ptr, DavimOperandStack*);
typedef int8_t (*davim_stack_push_proto)(func_cb_ptr, DavimOperandStack*, void*);
typedef void* (*davim_stack_pop_proto)(func_cb_ptr, DavimOperandStack*);
typedef void (*davim_stack_reset_proto)(func_cb_ptr, DavimOperandStack*);

//! The state of the stack module
typedef struct {
	sos_pid_t pid;
} stack_state_t;


#endif

