/*
 * Copyright (c) 2006, Wouter Horré
 * All rights reserved
 * 
 * See LICENSE for details
 */
#ifndef _DAVIM_VM_H_
#define _DAVIM_VM_H_

// needed typedefs
typedef uint8_t DavimContextId;
typedef uint32_t DavimVMVersion;
typedef uint32_t DavimContextBitmask;
typedef struct DavimVM DavimVM;
typedef struct DavimVMDescription DavimVMDescription;
typedef struct DavimContextHashEntry DavimContextHashEntry;
typedef struct DavimLock DavimLock;

#include "../davim.h"
#include "context.h"
#include "../operationstore/library_manager.h"
#include "vm_manager.h"

#define OPCODE_BITS 5
#define LIBRARY_MASK 0x07
#define OPCODE_MASK 0x1F

#define REBOOT_CONTEXT_ID 0
#define TIMER0_CONTEXT_ID 1

#define DAVIM_VM_INVALID 0xff

struct DavimContextHashEntry {
	DavimContext * context;
	DavimContextHashEntry * next;
} __attribute__((packed));

struct DavimLock{
  DavimContext* holder;
}  __attribute__((packed));

//! Structure representing a VM in DAViM
struct DavimVM
{
	DavimVMId id;
//	DavimVMVersion version;
	uint8_t halted : 1;
	DavimContextHashEntry * contexts[8];		//! The contexts running in this vm
	DavimVariable* globals[DAVIM_GLOBAL_COUNT];		//! The global variables of the VM
	DavimLock locks[DAVIM_GLOBAL_COUNT];			//! Which globals are locked?
	DavimLibraryId lib_mapping[8];				//! The mapping between global and local library ids
} __attribute__((packed));

struct DavimVMDescription {
	DavimVMId id;
//	DavimVMVersion version;
//	BOOL present;
	DavimLibraryId lib_mapping[8];
	DavimContextBitmask contexts;
} __attribute__((packed));

enum
{
	DAVIM_VM_NEW_FID,
	DAVIM_VM_FINALIZE_FID,
	DAVIM_VM_GET_LIBRARY_FID,
	DAVIM_VM_GET_OPCODE_FID,
	DAVIM_VM_GET_NEEDED_LIBS_FID,
	DAVIM_VM_LOCK_GLOBAL_FID,
	DAVIM_VM_UNLOCK_GLOBAL_FID,
	DAVIM_VM_IS_LOCKED_FID,
	DAVIM_VM_HALT_FID,
	DAVIM_VM_GET_DESCRIPTION_FID,
};

typedef DavimVM * (*davim_vm_new_proto)(func_cb_ptr,DavimVMId);
typedef void (*davim_vm_finalize_proto)(func_cb_ptr, DavimVM*);
typedef DavimLibraryId (*davim_vm_get_library_proto)(func_cb_ptr, DavimVMId, DavimOpCode);
typedef DavimOpCode (*davim_vm_get_opcode_proto)(func_cb_ptr, DavimOpCode);
typedef DavimLibraryBitmask (*davim_vm_get_needed_libs_proto)(func_cb_ptr, DavimVMId, DavimOpCode);
typedef int8_t (*davim_vm_lock_global_proto)(func_cb_ptr, DavimContext*, DavimVariableId);
typedef int8_t (*davim_vm_unlock_global_proto)(func_cb_ptr, DavimContext*, DavimVariableId);
typedef BOOL (*davim_vm_is_locked_proto)(func_cb_ptr, DavimVMId, DavimVariableId);
typedef void (*davim_vm_halt_proto)(func_cb_ptr, DavimVMId);
typedef void (*davim_vm_get_description_proto)(func_cb_ptr, DavimVMId, DavimVMDescription*);


//! The state of a DAViM virtual machine
typedef struct {
	func_cb_ptr concurrency_submit;
	func_cb_ptr concurrency_halt;
	func_cb_ptr context_new;
	func_cb_ptr context_finalize;
	func_cb_ptr context_reset;
	func_cb_ptr event_man_trigger_event;
	func_cb_ptr get_vm;
	sos_pid_t pid;
} vm_state_t;

#endif
