/* -*- Mode: C; tab-width:4 -*- */
/* ex: set ts=4 shiftwidth=4 softtabstop=4 cindent: */
/**
 * @brief Example module for SOS
 * This module shows some of concepts in SOS
 */

/**
 * Module needs to include <module.h>
 */
#include <module.h>
//#include <sos_timer.h>
//#include <random.h>
#include <led.h>

#include "blink_serv.h"

//#define BLINK_TIMER_INTERVAL	2048L
#define BLINK_TID               0
/**
 * Module can define its own state
 */
typedef struct {
    uint8_t pid;
    uint8_t state;
} app_state_t;

/**
 * Blink module
 *
 * @param msg Message being delivered to the module
 * @return int8_t SOS status message
 *
 * Modules implement a module function that acts as a message handler.  The
 * module function is typically implemented as a switch acting on the message
 * type.
 *
 * All modules should included a handler for MSG_INIT to initialize module
 * state, and a handler for MSG_FINAL to release module resources.
 */

static int8_t module(void *start, Message *e);

/**
 * This is the only global variable one can have.
 */
static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = DFLT_APP_ID0,
	.state_size     = sizeof(app_state_t),
    .num_timers     = 0,
	.num_sub_func   = 0,
	.num_prov_func  = 0,
	.module_handler = module,
};


static int8_t module(void *state, Message *msg)
{
	/**
	 * The module is passed in a void* that contains its state.  For easy
	 * reference it is handy to typecast this variable to be of the
	 * applications state type.  Note that since we are running as a module,
	 * this state is not accessible in the form of a global or static
	 * variable.
     */
    app_state_t *s = (app_state_t*)state;

    /**
     * Switch to the correct message handler
     */
    switch (msg->type){

        /**
         * MSG_INIT is used to initialize module state the first time the
         * module is used.  Many modules set timers at this point, so that
         * they will continue to receive periodic (or one shot) timer events.
         */
        case MSG_INIT:
		{
			s->pid = msg->did;
			s->state = 0;
		    /**
             * The timer API takes the following parameters:
             * - PID of the module the timer is for
             * - Timer ID (used to distinguish multiple timers of different
             *   ..types on the same host)
             * - Type of timer
             * - Timer delay in
             */
			DEBUG("Blink Serv Start\n");
			//ker_timer_init(s->pid, BLINK_TID, TIMER_REPEAT);
            //ker_timer_start(s->pid, BLINK_TID, BLINK_TIMER_INTERVAL);
			ker_led(LED_YELLOW_OFF);
			break;
		}


        /**
         * MSG_FINAL is used to shut modules down.  Modules should release all
         * resources at this time and take care of any final protocol
         * shutdown.
         */
        case MSG_FINAL:
        {
            /**
             * Stop the timer
             */
            //ker_timer_stop(s->pid, BLINK_TID);
			DEBUG("Blink Serv Stop\n");
			break;
        }


        /**
         * All timers addressed to this PID, regardless of the timer ID, are of
         * type MSG_TIMER_TIMEOUT and handled by this handler.  Timers with
         * different timer IDs can be further distinguished by testing for the
         * type, as demonstrated in the relay module.
         */
//        case MSG_TIMER_TIMEOUT:
//		{
//			MsgParam* params = (MsgParam*)(msg->data);
//
////			ker_led(LED_YELLOW_TOGGLE);
//			if (params->byte == BLINK_TID)
//			{
//
//				if(s->state == 1){
//					/**
//					  * sys_led is SOS system call for LED.
//					  */
//					ker_led(LED_YELLOW_OFF);
//					DEBUG("Blink Yellow OFF\n");
//				}
//				else{
//					ker_led(LED_YELLOW_ON);
//					DEBUG("Blink Yellow ON\n");
//				}
//				s->state++;
//				if(s->state > 1) s->state = 0;
//			}
//			break;
//		}
		case MY_BLINK_MSG_STATE:
		{
				MsgParam* params = (MsgParam*)(msg->data);
				s->state = params->byte;
				if(s->state == 1){
					ker_led(LED_YELLOW_OFF);
					DEBUG("Blink Serv Yellow OFF\n");
  			}
				else{
					ker_led(LED_YELLOW_ON);
					DEBUG("Blink Serv Yellow ON\n");
				}
			
		}

        /**
         * The default handler is used to catch any messages that the module
         * does no know how to handle.
         */
        default:
			return -EINVAL;
	}

    /**
     * Return SOS_OK for those handlers that have successfully been handled.
     */
    return SOS_OK;
}

#ifndef _MODULE_
mod_header_ptr blink_serv_get_header()
{
    return sos_get_header_address(mod_header);
}
#endif

