/**
 * Module needs to include <module.h>
 */
#include <module.h>
#include <sos_timer.h>

#include "blink_client.h"
#include "../blink_serv/blink_serv.h"

/**
 * Module can define its own state
 */
typedef struct {
	func_cb_ptr blink_serv_set_led;
	sos_pid_t pid;
	uint8_t state;
} blink_client_state_t;

/*
 * Forward declaration of module 
 */
static int8_t blink_client(void *start, Message *e);

/**
 * This is the only global variable one can have.
 */
static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = BLINK_CLIENT_PID,
	.state_size     = sizeof(blink_client_state_t),
    .num_timers     = 1,
	.num_sub_func   = 1,
	.num_prov_func  = 0,
	.module_handler = blink_client,
	.funct = {
		[0] = { NULL, "cCC2", BLINK_SERV_PID, BLINK_SERV_SET_LED_ID }
	},
};


static int8_t blink_client(void *state, Message *msg)
{
    blink_client_state_t *s = (blink_client_state_t *) state;

    /**
     * Switch to the correct message handler
     */
    switch (msg->type){

        case MSG_INIT:
		{
			DEBUG("BLINK CLIENT START\n");
			s->pid = msg->did;
			
			s->state = 0;
			ker_timer_init(s->pid,BLINK_CLIENT_TID,TIMER_REPEAT);
			ker_timer_start(s->pid,BLINK_CLIENT_TID,BLINK_CLIENT_TIMER_INTERVAL);
			return SOS_OK;
		}


        case MSG_FINAL:
        {
			DEBUG("BLINK CLIENT STOP\n");
			ker_timer_release(s->pid,BLINK_CLIENT_TID);
			return SOS_OK;
        }

		case MSG_TIMER_TIMEOUT:
		{
			MsgParam* params = (MsgParam*)(msg->data);

			if(params->byte == BLINK_CLIENT_TID){
				DEBUG("BLINK CLIENT TIMER TIMEOUT: STATE %i\n",s->state);
				switch (s->state) {
					case 0:
						SOS_CALL(s->blink_serv_set_led, blink_serv_set_led_proto, BLINK_LED_RED, BLINK_LED_ON);
						break;
					case 1:
						SOS_CALL(s->blink_serv_set_led, blink_serv_set_led_proto, BLINK_LED_GREEN, BLINK_LED_ON);
						break;
					case 2:
						SOS_CALL(s->blink_serv_set_led, blink_serv_set_led_proto, BLINK_LED_YELLOW, BLINK_LED_ON);
						break;
					case 3:
						SOS_CALL(s->blink_serv_set_led, blink_serv_set_led_proto, BLINK_LED_RED, BLINK_LED_OFF);
						break;
					case 4:
						SOS_CALL(s->blink_serv_set_led, blink_serv_set_led_proto, BLINK_LED_GREEN, BLINK_LED_OFF);
						break;
					case 5:
						SOS_CALL(s->blink_serv_set_led, blink_serv_set_led_proto, BLINK_LED_YELLOW, BLINK_LED_OFF);
						break;
					default:
						return -EINVAL;
				}
				s->state++;
				s->state %= 6;
			}
			return SOS_OK;
		}


        default:
			return -EINVAL;
	}

    /**
     * Return SOS_OK for those handlers that have successfully been handled.
     */
    return SOS_OK;
}

#ifndef _MODULE_
mod_header_ptr blink_client_get_header()
{
    return sos_get_header_address(mod_header);
}
#endif

