/**
 * Module needs to include <module.h>
 */
#include <module.h>
#include <led.h>

#include "blink_serv.h"

/**
 * Module can define its own state
 */
typedef struct {
	sos_pid_t pid;
} blink_serv_state_t;

static int8_t blink_serv_set_led(func_cb_ptr p, uint8_t led, uint8_t state);
static int8_t blink_serv(void *start, Message *e);


/**
 * This is the only global variable one can have.
 */
static mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = BLINK_SERV_PID,
	.state_size     = sizeof(blink_serv_state_t),
    .num_timers     = 0,
	.num_sub_func   = 0,
	.num_prov_func  = 1,
	.module_handler = blink_serv,
	.funct = {
		[0] = { blink_serv_set_led, "cCC2", BLINK_SERV_PID, BLINK_SERV_SET_LED_ID },
	},

};


static int8_t blink_serv(void *state, Message *msg)
{
    blink_serv_state_t *s = (blink_serv_state_t *) state;

    /**
     * Switch to the correct message handler
     */
    switch (msg->type){

        case MSG_INIT:
		{
			DEBUG("BLINK SERV START\n");
			s->pid = msg->did;
			
			ker_led(LED_RED_OFF);
			ker_led(LED_GREEN_OFF);
			ker_led(LED_YELLOW_OFF);

			return SOS_OK;
		}


        case MSG_FINAL:
        {
			DEBUG("BLINK SERV STOP\n");

			return SOS_OK;
        }


        default:
			return -EINVAL;
	}

    /**
     * Return SOS_OK for those handlers that have successfully been handled.
     */
    return SOS_OK;
}

static int8_t blink_serv_set_led(func_cb_ptr p, uint8_t led, uint8_t state) {
	switch (led) {
		case BLINK_LED_RED:
		{
			if(state == BLINK_LED_OFF) {
				DEBUG("BLINK SERV RED LED OFF\n");
				ker_led(LED_RED_OFF);
			} else if(state == BLINK_LED_ON) {
				DEBUG("BLINK SERV RED LED ON\n");
				ker_led(LED_RED_ON);
			}
			break;
		}
		case BLINK_LED_GREEN:
		{
			if(state == BLINK_LED_OFF) {
				DEBUG("BLINK SERV GREEN LED OFF\n");
				ker_led(LED_GREEN_OFF);
			} else if(state == BLINK_LED_ON) {
				DEBUG("BLINK SERV GREEN LED ON\n");
				ker_led(LED_GREEN_ON);
			}
			break;
		}
		case BLINK_LED_YELLOW:
		{
			if(state == BLINK_LED_OFF) {
				DEBUG("BLINK SERV YELLOW LED OFF\n");
				ker_led(LED_YELLOW_OFF);
			} else if(state == BLINK_LED_ON) {
				DEBUG("BLINK SERV YELLOW LED ON\n");
				ker_led(LED_YELLOW_ON);
			}
			break;
		}
		default:
			return -EINVAL;
	}
	return SOS_OK;
}

#ifndef _MODULE_
mod_header_ptr blink_serv_get_header()
{
    return sos_get_header_address(mod_header);
}
#endif

